$! 'f$verify(0)'
$!			WWWsearch.COM
$!
$!	14-MAR-1995	F.Macrides (macrides@sci.wfbr.edu)
$!			ISINDEX search script interface to the
$!			VMSIndex search engine (query.exe).
$!			For use with CERN and OSU http servers.
$!			(** Use WWWsearch.COM-dt, renamed to .COM, for
$!			    versions of the OSU server before v1.8a **)
$!
$!			Should be placed where your ISINDEX search script
$!			has been mapped in the server's configuration
$!			file, e.g.,
$!			  for CERN:	Search	/HTTPD_Dir/WWWsearch.COM
$!			  for OSU:	Search	WWW_Root:[bin]WWWsearch.com
$!
$!			It allows you to specify a location of
$!			indexes (.IDX, .SEL and .POS files), and set
$!			up symbols for features of the hit lists, in
$!			calls to WWWquery (query/www).  You can further
$!			elaborate this model script so that it
$!			tests the "database" symbol and sets the
$!			hit list features as a function of the index
$!			(e.g., don't set WWW_SEND_RAW_FILES if the
$!			index is for a sectional database), and/or
$!			substitutes databases whose names differ from
$!			the root of the ISINDEX document.
$!
$!	13-APR-1995	Added WWW_VMSINDEX_TITLE string. - FM
$!
$!	25-APR-1995	Modified to invoke UserSearch.COM for /~username/
$!			URLs (can't be made to work for OSU server through
$!			v1.8; see comments below).  Added symbols for
$!			converting translated paths in the the .SEL files
$!			back to HTTP logical paths in the hit lists. - FM
$!
$!	26-APR-1995	Found a way to make it work with the OSU server. - FM
$!
$!	28-APR-1995     The cgi_init() bug was fixed in v1.8a of the
$!			OSU server, so the external fix was removed
$!			from this script. - FM
$!
$!	13-JAN-1996     Updated for query.c v2.4 or later. - FM
$!
$!	01-FEB-1996	Added WWW_HITS_TARGET to the Hit List Options
$!			which if non-NULL and a non-zero-length string
$!			will cause TARGET="string" to be added to the
$!			hit list anchors. - FM
$!
$!	04-MAY-1996	If WWW_PATH_TRANSLATED is blank, use
$!			WWW_PATH_INFO - BT
$!
$!----------------------------------------------------------------------------
$!
$!	Query.c is the search engine developed by Bruce Tanner and Foteos
$!	Macrides for accessing foo.doc;*, foo.IDX;*, foo.SEL;*, and foo.POS
$!	filesets.  The doc file(s) is(are) indexed with BUILD_INDEX.EXE,
$!	yielding the IDX, SEL and POS filesets.
$!
$!	This search engine was develeped by Bruce and colleagues associated
$!	with the VMSGopher-L@trln.lib.unc.edu list for use with the
$!	VMSGopherServer, and was extended by Foteos and Bruce for use with
$!	http servers.  It is similar to WAIS, but uses native indexing
$!	facilities of the VMS Record Management System (RMS).
$!
$!	Build BUILD_INDEX.EXE and QUERY.EXE by executing MAKE.COM.  It will
$!	check out your system and then use MMS or command files, and the
$!	appropriate foo.OPT files.  QUERY.EXE is executed with its /www
$!	qualifier for http servers, normally via a WWWquery foreign command
$!	which includes the qualifier.  The symbol WWW_GATEWAY_INTERFACE,
$!	with is set for CGI scripts by http servers, also acts as a /www
$!	qualifier for query.exe.
$!
$!	Read BUILD_INDEX.DOC for information about the indexer.
$!
$!	The example indexdemo.html ISINDEX document is for use with this
$!	script and an index of all HTML documents (i.e., which have extension
$!	.html) in your WWW_Root data tree.  The index should have been
$!	created with the example HTTP_INDEXTREE.COM in this distribution,
$!	and reside in the WWW_Root:[Index] subdirectory.  If instead of one,
$!	"whole tree" index, you built a series of indexes for components of
$!	your data tree, you could call WWWquery with a wildcarded index spec
$!	to search them all, or particular combinations which match the
$!	wildcarding.  Do that substitution where this script presently
$!	substitutes "VMSINDEX" for "INDEXDEMO" as the database symbol.
$!
$!	The symbol WWW_SEND_RAW_FILE must be made non-NULL (we set it equal
$!	to "YES", below, but any string will do, e.g., "TRUE"), so that
$!	WWWquery will not add html "packaging", as the .html file should
$!	already have a <HEAD> section with <TITLE>.
$!
$!	Note that the symbol WWW_OMIT_PRE_TAG, if defined, would cause
$!	the <PRE> tag to be omitted by WWWquery, but not the other html
$!	"packaging".  You want it all omitted, since you presumably
$!	indexed sets of complete, normally independent foo.html files.
$!
$!	The symbol WWW_SHOW_FILETYPE can be made non-NULL (e.g., "YES") to
$!	have icons (or ALT text) indicating the file types to be inserted
$!	into the hit lists returned from searches.
$!
$!	If the symbol WWW_MAXIMUM_HITS is made non-NULL (i.e., set equal to
$!	a number), then the hit lists returned by searches will be abridged,
$!	if necessary, so as not to exceed that number.  The WWW_STARTING_HIT
$!	symbol can be set to a number to indicate the starting number in hit
$!	lists.
$!
$!	The symbol WWW_SHOW_SIZE can be made non-NULL (e.g., "YES") to have
$!	the size of each file or database section indicated in the hit lists.
$!
$!	The symbol WWW_SHOW_DATE can be made non-NULL to have the date of the
$!	database indicated in the hit lists (DD-MMM-YY).  If WWW_SHOW_TIME
$!	also is made non-NULL, for the current year the hour and minute will
$!	be indicated instead of the year (DD-MMM HH:MM).  If WWW_USE_MDATE is
$!	made non-NULL, the date of the last modification (e.g., from an
$!	APPEND) will be used.  The default is the file creation date (i.e.,
$!	of the highest version).
$!
$!	The symbol WWW_VMSINDEX_TITLE can be set to a string for the TITLE
$!	tag in hit lists and section fetches.  Otherwise, the TITLE defaults
$!	to "VMS Indexed Database Search".
$!
$!	The symbol WWW_SEND_CONTENTS can be made non-NULL (e.g., "YES") to
$!	have searches which yield only one hit return that document (instead
$!	of a "hit list" with a link for the document).  This can be used to
$!	fetch particular documents via unique identifiers in them, or other
$!	query known to generate one hit, for that document.
$!
$!	The symbol WWW_SORT can be made non-NULL (e.g., "YES") to invoke
$!	sorting of hits by score, and the symbol WWW_SHOW_SCORE can be made
$!	non-NULL to include the scores in hit list titles.  Setting these
$!	symbols is equivalent to calling query.exe with /sort and /score.
$!
$!	See the header of query.c for additional regulatory symbols.
$!
$!	Set up the foreign command definition for WWWquery, below (or define
$!	it in the CERN server's SpawnInit.COM).  This script must be in the
$!	server's htbin directory, but query.exe can be anywhere that is
$!	accessible to the server (e.g., where you broke out and built the
$!	VMSIndex distribution).  Check the default document directory and
$!	translation symbols, below (see header of query.c for more info).
$!
$!	For the OSU server, you also must have Set_DCL_Env.EXE in its htbin
$!      directory.
$!
$!----------------------------------------------------------------------------
$! WWWquery := "$device:[directory]query/www"
$!-------------------------------------------
$ INDEX_DIR = "WWW_Root:[Index]"	! Location of server data tree indexes
$!----------------------------------------------------------------------------
$ crlf = f$fao("!/") 
$!
$ If f$trnlnm("WWW_OUT") .nes. ""
$ Then						! CERN server
$    WWW_IPATH_INFO = "/www/"			! Default Document Directory
$    WWW_IPATH_TRANSLATED = "/WWW_Root/000000/"	! Default Translation
$    say = "write WWW_OUT"
$    put = "create WWW_OUT"
$    define_output   = "!"
$    deassign_output = "!"
$ Else						! OSU server
$    WWW_IPATH_INFO = "/www/"			! Default Document Directory
$    WWW_IPATH_TRANSLATED = "/WWW_Root/documents/" ! Default Translation
$    say = "write net_link"
$    put = "create net_link"
$    define_output   = "define/nolog sys$output net_link"
$    deassign_output = "deassign sys$output"
$    say "<DNETRECMODE>"		! Set implied carriage control
$    mcr 'f$parse("SET_DCL_ENV.EXE;",f$environment("PROCEDURE")) "WWW_"
$ EndIf
$!----------------------------------------------------------------------------
$ If WWW_REQUEST_METHOD .eqs. "HEAD"	! Request for header only.
$ Then
$   say "Content-type: text/html", crlf	! CGI header.
$   goto done
$ Endif
$!----------------------------------------------------------------------------
$!
$! If it's a /~username/ URL, use UserSearch.com
$!
$ If f$locate("/~",WWW_PATH_INFO) .eq. 0
$ Then
$   @'f$parse("USERSEARCH.COM;",f$environment("PROCEDURE"))'
$   goto done
$ EndIf
$!----------------------------------------------------------------------------
$!
$! WWW_PATH_TRANSLATED has the database name as .../database.html
$!
$ ISINDEX_URL = f$edit(WWW_PATH_TRANSLATED,"COLLAPSE,UPCASE")
$!
$! and if this script is called with a raw database name
$! e.g. <isindex action="/htbin/wwwsearch/database">
$! then WWW_PATH_TRANSLATED will be blank and WWW_PATH_INFO has the database
$!
$ if ISINDEX_URL .eqs. "" then -
    ISINDEX_URL = f$edit(WWW_PATH_INFO,"COLLAPSE,UPCASE")
$ gosub EXTRACT_DATABASE		! Strip it to the database root.
$!----------------------------------------------------------------------------
$!
$! Check the database name symbol for ones we want to change. Otherwise,
$! we assume the database has the same name as the ISINDEX document.
$! Also can set the title as a function of the index, and change the
$! WWW_IPATH_INFO and WWW_IPATH_TRANSLATED symbols here if the defaults
$! don't apply to particular indexes.
$!
$ if database .eqs. "INDEXDEMO"
$ then
$   database = "VMSINDEX"
$   WWW_VMSINDEX_TITLE = "ISINDEX Search Demonstration"
$ endif
$!
$! Add the location of indexes.
$!
$ database = INDEX_DIR + database
$!
$! Hit list options:
$! =================
$  WWW_SEND_RAW_FILE = "YES"	! Send direct URL's without a Range
$! WWW_ITEMS_FOUND  = "There are <EM>%d</EM> item(s) found" !Default string
$  WWW_ITEMS_FOUND  = "<EM>%d</EM> item(s) were found" !We'll use this string
$! WWW_ITEMS_LISTED = "Listing <EM>%d</EM> to <EM>%d</EM>"  !Default string
$  WWW_SHOW_FILETYPE = "YES"	! Show filetype icons or ALT strings
$  WWW_MAXIMUM_HITS = 250	! Limit number of hits to return
$  WWW_STARTING_HIT = 1		! Start listing with the first hit.
$  WWW_SHOW_SIZE = "YES"	! Bytes or KB
$! WWW_SHOW_DATE = "YES"	! DD-MMM-YY
$! WWW_SHOW_TIME = "YES"	! Replace -YY with HH:MM for current year
$! WWW_USE_MDATE = "YES"	! Use last modified date (default is creation)
$! WWW_SORT = "YES"		! Sort according to the score.
$! WWW_SHOW_SCORE = "YES"	! Show the score.
$! WWW_HITS_TARGET = "foo"	! Add TARGET attribute to anchors.
$!
$! Document options:
$! =================
$! WWW_OMIT_PRE_TAG = "YES"	! See header of WWWquery.c
$! WWW_DATADIRECTORY = "foo"	! See header of WWWquery.c
$! WWW_FORBIDDEN = "blah"	! See header of WWWquery.c
$! WWW_VMSINDEX_TITLE = "Wow"	! See header of WWWquery.c
$!
$! Script options:
$! ===============
$  WWW_MINIMUM_WORD = "3"	! Ignore keywords smaller than this number.
$  WWW_SEND_CONTENTS = "YES"	! Send Location: redirect if only one hit.
$! WWW_OMIT_CONTENT_TYPE = "YES"	! See header of WWWquery.c
$! WWW_OMIT_HEAD = "YES"		! See header of WWWquery.c
$! WWW_OMIT_FOOT = "YES"		! See header of WWWquery.c
$! WWW_VIA_FORM = "YES"			! See header of query.c
$! WWW_ADD_CONTEXT = "YES"		! See header of query.c
$! WWW_END_FORM = "YES"			! See header of query.c
$! WWW_OMIT_IPATH_CONVERSION = "YES"	! See header of query.c
$!
$!----------------------------------------------------------------------------
$!
$! Reconstruct the query (unescaped).
$!
$ query = WWW_KEY_1
$ num = 2
$ Loop:
$ If (WWW_KEY_COUNT - num) .ge. 0 
$ Then
$   query = query + " " + WWW_KEY_'num'
$   num = num + 1
$   goto Loop
$ EndIf
$!----------------------------------------------------------------------------
$!
$! Do the search or RANGE fetch.
$!
$ define_output
$ WWWquery "''database'" "''query'"
$ deassign_output
$!
$done:
$ exit
$!
$!***************** EXTRACT_DATABASE SUBROUTINE ******************************
$! Extract database symbol from ISINDEX_URL symbol
$!
$ EXTRACT_DATABASE:
$ database = ISINDEX_URL
$ trimming_loop:
$ offset = f$locate("/",database) + 1
$ If offset .le. f$length(database)
$ Then
$   database = f$extract(offset,f$length(database)-offset,database)
$   goto trimming_loop
$ Else
$   database = f$extract(0,f$locate(".",database),database)
$ Endif
$ RETURN   
$!************************ END EXTRACT_DATABASE ******************************
